@@ -5,7 +5,10 @@
                         transform = "none",
                         formula, ##type = c("wa","lc"),
                         scaling = 3, rank = "full",
-                        model = c("CCA", "CA"), ...) {
+                        ##model = c("CCA", "CA"),
+                        join = "left",
+                        ...) {
+    origX <- X ## store for later
     namX <- deparse(substitute(X))
     namP <- deparse(substitute(passive))
     ## Apply a transformation - let tran deal with arg matching
@@ -14,7 +17,7 @@
         passive <- tran(passive, method = transform, ...)
     }
     ## merge X and passive
-    dat <- join(X, passive, type = "left")
+    dat <- join(X, passive, type = join)
     X <- dat[[1]]
     passive <- dat[[2]]
     ## common set of species
@@ -22,8 +25,8 @@
     X <- X[, tmp]
     passive <- passive[, tmp]
     ## check what type of ordination is required
-    if(isTRUE(missing(method)))
-        method <- "cca"
+    ##if(isTRUE(missing(method)))
+    ##    method <- "cca"
     method <- match.arg(method)
     FUN <- match.fun(method)
     ## if no env do unconstrained
@@ -56,9 +59,9 @@
         }
     }
     ## process predict args
-    if(isTRUE(missing(model)))
-        model <- "CCA"
-    model <- match.arg(model)
+    ##if(isTRUE(missing(model)))
+    ##    model <- "CCA"
+    ##model <- match.arg(model)
     ## fitted values for passive
     pred <- predict(ord, newdata = passive, type = "wa",
                     scaling = scaling, model = "CCA", rank = rank)
@@ -69,10 +72,11 @@
     ## return object
     res <- list(ordination = ord, fitted.values = pred,
                 method = method, formula = formula, #type = type,
-                scaling = scaling, rank = rank, model = model,
-                labels = nams, call = match.call())
+                scaling = scaling, rank = rank, ##model = model,
+                labels = nams, call = match.call(),
+                X = origX, transform = transform)
     class(res) <- "timetrack"
-    return(res)
+    res
 }
 
 `print.timetrack` <- function(x, ...) {
